@@ -18,17 +18,21 @@
 
 #define MIR_LOG_COMPONENT "event-access"
 
+#include "handle_event_exception.h"
+
 #include "mir/event_type_to_string.h"
 #include "mir/log.h"
 
 #include "mir_toolkit/events/event.h"
 #include "mir/events/event_private.h"
+#include "mir/events/surface_placement_event.h"
 
 #include "mir_toolkit/events/surface_event.h"
 #include "mir_toolkit/events/resize_event.h"
 #include "mir_toolkit/events/prompt_session_event.h"
 #include "mir_toolkit/events/orientation_event.h"
 #include "mir_toolkit/events/input_device_state_event.h"
+#include "mir_toolkit/events/surface_placement.h"
 
 #include <cstdlib>
 #include <cstring>
@@ -38,15 +42,16 @@ namespace ml = mir::logging;
 namespace
 {
 template <typename EventType>
-void expect_event_type(EventType const* ev, MirEventType t)
+void expect_event_type(EventType const* ev, MirEventType t) MIR_HANDLE_EVENT_EXCEPTION(
 {
-    if (ev->type() != t)
+    auto type = ev->type();
+    if (type != t)
     {
         mir::log_critical("Expected " + mir::event_type_to_string(t) + " but event is of type " +
-            mir::event_type_to_string(ev->type()));
+            mir::event_type_to_string(type));
         abort();
     }
-}
+})
 
 void expect_index_in_range(size_t size, size_t index)
 {
@@ -86,29 +91,31 @@ std::string mir::event_type_to_string(MirEventType t)
 }
 
 
-MirEventType mir_event_get_type(MirEvent const* ev)
+MirEventType mir_event_get_type(MirEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
-    switch (ev->type())
+    auto type = ev->type();
+    switch (type)
     {
     case mir_event_type_key:
     case mir_event_type_motion:
         return mir_event_type_input;
     default:
-        return ev->type();
+        return type;
     }
-}
+})
 
-MirInputEvent const* mir_event_get_input_event(MirEvent const* ev)
+MirInputEvent const* mir_event_get_input_event(MirEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
-    if (ev->type() != mir_event_type_key && ev->type() != mir_event_type_motion)
+    auto type = ev->type();
+    if (type != mir_event_type_key && type != mir_event_type_motion)
     {
         mir::log_critical("Expected input event but event is of type " +
-            mir::event_type_to_string(ev->type()));
+            mir::event_type_to_string(type));
         abort();
     }
 
     return ev->to_input();
-}
+})
 
 MirSurfaceEvent const* mir_event_get_surface_event(MirEvent const* ev)
 {
@@ -152,12 +159,15 @@ MirKeymapEvent const* mir_event_get_keymap_event(MirEvent const* ev)
     return ev->to_keymap();
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 MirInputConfigurationEvent const* mir_event_get_input_configuration_event(MirEvent const* ev)
 {
     expect_event_type(ev, mir_event_type_input_configuration);
 
     return ev->to_input_configuration();
 }
+#pragma GCC diagnostic pop
 
 MirSurfaceOutputEvent const* mir_event_get_surface_output_event(MirEvent const* ev)
 {
@@ -175,184 +185,199 @@ MirInputDeviceStateEvent const* mir_event_get_input_device_state_event(MirEvent
 
 /* Surface event accessors */
 
-MirSurfaceAttrib mir_surface_event_get_attribute(MirSurfaceEvent const* ev)
+MirSurfaceAttrib mir_surface_event_get_attribute(MirSurfaceEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_surface);
 
     return ev->attrib();
-}
+})
 
-int mir_surface_event_get_attribute_value(MirSurfaceEvent const* ev)
+int mir_surface_event_get_attribute_value(MirSurfaceEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_surface);
 
     return ev->value();
-}
+})
 
 /* Resize event accessors */
 
-int mir_resize_event_get_width(MirResizeEvent const* ev)
+int mir_resize_event_get_width(MirResizeEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_resize);
     return ev->width();
-}
+})
 
-int mir_resize_event_get_height(MirResizeEvent const* ev)
+int mir_resize_event_get_height(MirResizeEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_resize);
     return ev->height();
-}
+})
 
 /* Prompt session event accessors */
 
-MirPromptSessionState mir_prompt_session_event_get_state(MirPromptSessionEvent const* ev)
+MirPromptSessionState mir_prompt_session_event_get_state(MirPromptSessionEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_prompt_session_state_change);
     return ev->new_state();
-}
+})
 
 /* Orientation event accessors */
 
-MirOrientation mir_orientation_event_get_direction(MirOrientationEvent const* ev)
+MirOrientation mir_orientation_event_get_direction(MirOrientationEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_orientation);
     return ev->direction();
-}
+})
 
 /* Keymap event accessors */
 
-void mir_keymap_event_get_keymap_buffer(MirKeymapEvent const* ev, char const** buffer, size_t* length)
+void mir_keymap_event_get_keymap_buffer(MirKeymapEvent const* ev, char const** buffer, size_t* length) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_keymap);
 
     *buffer = ev->buffer();
     *length = ev->size();
-}
+})
 
-MirInputDeviceId mir_keymap_event_get_device_id(MirKeymapEvent const* ev)
+MirInputDeviceId mir_keymap_event_get_device_id(MirKeymapEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_keymap);
 
     return ev->device_id();
-}
+})
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 /* Input configuration event accessors */
-
-MirInputConfigurationAction mir_input_configuration_event_get_action(MirInputConfigurationEvent const* ev)
+MirInputConfigurationAction mir_input_configuration_event_get_action(MirInputConfigurationEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_configuration);
     return ev->action();
-}
+})
 
-int64_t mir_input_configuration_event_get_time(MirInputConfigurationEvent const* ev)
+int64_t mir_input_configuration_event_get_time(MirInputConfigurationEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_configuration);
     return ev->when().count();
-}
+})
 
-MirInputDeviceId mir_input_configuration_event_get_device_id(MirInputConfigurationEvent const* ev)
+MirInputDeviceId mir_input_configuration_event_get_device_id(MirInputConfigurationEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_configuration);
     return ev->id();
-}
+})
+#pragma GCC diagnostic pop
 
 /* Surface output event accessors */
 
-int mir_surface_output_event_get_dpi(MirSurfaceOutputEvent const* ev)
+int mir_surface_output_event_get_dpi(MirSurfaceOutputEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_surface_output);
     return ev->dpi();
-}
+})
 
-MirFormFactor mir_surface_output_event_get_form_factor(MirSurfaceOutputEvent const* ev)
+MirFormFactor mir_surface_output_event_get_form_factor(MirSurfaceOutputEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_surface_output);
     return ev->form_factor();
-}
+})
 
-float mir_surface_output_event_get_scale(MirSurfaceOutputEvent const* ev)
+float mir_surface_output_event_get_scale(MirSurfaceOutputEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_surface_output);
     return ev->scale();
-}
+})
 
-uint32_t mir_surface_output_event_get_output_id(MirSurfaceOutputEvent const *ev)
+double mir_surface_output_event_get_refresh_rate(MirSurfaceOutputEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
+{
+    expect_event_type(ev, mir_event_type_surface_output);
+    return ev->refresh_rate();
+})
+
+uint32_t mir_surface_output_event_get_output_id(MirSurfaceOutputEvent const *ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_surface_output);
     return ev->output_id();
-}
+})
 
-MirPointerButtons mir_input_device_state_event_pointer_buttons(MirInputDeviceStateEvent const* ev)
+MirPointerButtons mir_input_device_state_event_pointer_buttons(MirInputDeviceStateEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     return ev->pointer_buttons();
-}
+})
 
-float mir_input_device_state_event_pointer_axis(MirInputDeviceStateEvent const* ev, MirPointerAxis axis)
+float mir_input_device_state_event_pointer_axis(MirInputDeviceStateEvent const* ev, MirPointerAxis axis) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     return ev->pointer_axis(axis);
-}
+})
 
-int64_t mir_input_device_state_event_time(MirInputDeviceStateEvent const* ev)
+int64_t mir_input_device_state_event_time(MirInputDeviceStateEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     return ev->when().count();
-}
+})
 
-MirInputEventModifiers mir_input_device_state_event_modifiers(MirInputDeviceStateEvent const* ev)
+MirInputEventModifiers mir_input_device_state_event_modifiers(MirInputDeviceStateEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     return ev->modifiers();
-}
+})
 
-uint32_t mir_input_device_state_event_device_count(MirInputDeviceStateEvent const* ev)
+uint32_t mir_input_device_state_event_device_count(MirInputDeviceStateEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     return ev->device_count();
-}
+})
 
-MirInputDeviceId mir_input_device_state_event_device_id(MirInputDeviceStateEvent const* ev, uint32_t index)
+MirInputDeviceId mir_input_device_state_event_device_id(MirInputDeviceStateEvent const* ev, uint32_t index) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     expect_index_in_range(ev->device_count(), index);
     return ev->device_id(index);
-}
+})
 
-uint32_t const* mir_input_device_state_event_device_pressed_keys(MirInputDeviceStateEvent const* ev, uint32_t index)
+uint32_t mir_input_device_state_event_device_pressed_keys_for_index(MirInputDeviceStateEvent const* ev,
+                                                                    uint32_t index,
+                                                                    uint32_t pressed_index) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     expect_index_in_range(ev->device_count(), index);
-    return ev->device_pressed_keys(index);
-}
+    return ev->device_pressed_keys_for_index(index, pressed_index);
+})
 
-uint32_t mir_input_device_state_event_device_pressed_keys_count(MirInputDeviceStateEvent const* ev, uint32_t index)
+uint32_t mir_input_device_state_event_device_pressed_keys_count(MirInputDeviceStateEvent const* ev, uint32_t index) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     expect_index_in_range(ev->device_count(), index);
     return ev->device_pressed_keys_count(index);
-}
+})
 
-MirPointerButtons mir_input_device_state_event_device_pointer_buttons(MirInputDeviceStateEvent const* ev, uint32_t index)
+MirPointerButtons mir_input_device_state_event_device_pointer_buttons(MirInputDeviceStateEvent const* ev, uint32_t index) MIR_HANDLE_EVENT_EXCEPTION(
 {
     expect_event_type(ev, mir_event_type_input_device_state);
     expect_index_in_range(ev->device_count(), index);
     return ev->device_pointer_buttons(index);
-}
+})
+
+MirSurfacePlacementEvent const* mir_event_get_surface_placement_event(MirEvent const* event) MIR_HANDLE_EVENT_EXCEPTION(
+{
+    return event->to_surface_placement();
+})
+
+MirRectangle mir_surface_placement_get_relative_position(MirSurfacePlacementEvent const* event) MIR_HANDLE_EVENT_EXCEPTION(
+{
+    return event->placement();
+})
 
 // TODO: Until we opaquify the MirEvent structure and add
 // a ref count ref is implemented as copy.
-MirEvent const* mir_event_ref(MirEvent const* ev)
+MirEvent const* mir_event_ref(MirEvent const* ev) MIR_HANDLE_EVENT_EXCEPTION(
 {
-    return ev->clone();
-}
+    return new MirEvent(*ev);
+})
 
 void mir_event_unref(MirEvent const* ev)
 {
-    if (mir_event_get_type(ev) == mir_event_type_keymap)
-    {
-        const_cast<MirEvent*>(ev)->to_keymap()->free_buffer();
-    }
-
     delete const_cast<MirEvent*>(ev);
 }
