@@ -685,6 +685,10 @@ bool msh::CanonicalWindowManagerPolicy::handle_touch_event(MirTouchEvent const*
 
         case mir_touch_action_change:
             continue;
+
+        case mir_touch_actions:
+            abort();
+            return false;
         }
     }
 
